@@ -217,8 +217,13 @@ impl Initial {
     }
     #[cfg(feature = "extended-value")]
     #[inline]
-    pub async fn extend_config(&mut self, timeout: Duration, base: &Path) {
-        self.config = self.config.extend(timeout, base).await;
+    pub async fn extend_config(&mut self, timeout: Duration, base: &Path) -> EResult<()> {
+        self.config = if let Some(config) = self.config.take() {
+            Some(config.extend(timeout, base).await?)
+        } else {
+            None
+        };
+        Ok(())
     }
     #[inline]
     pub fn workers(&self) -> u32 {
