@@ -11,6 +11,10 @@ use std::convert::{TryFrom, TryInto};
 use std::fmt;
 use std::hash::{BuildHasher, Hash, Hasher};
 use std::iter::FromIterator;
+#[cfg(feature = "extended-value")]
+use std::path::Path;
+#[cfg(feature = "extended-value")]
+use std::time::Duration;
 
 pub use de::*;
 pub use ser::*;
@@ -462,6 +466,96 @@ impl Value {
             _ => false,
         }
     }
+
+    #[cfg(feature = "extended-value")]
+    pub async fn extend(self, timeout: Duration, base: &Path) -> EResult<Value> {
+        let op = crate::op::Op::new(timeout);
+        extend_value(self, &op, base).await
+    }
+}
+
+#[cfg(feature = "extended-value")]
+#[async_recursion::async_recursion]
+async fn extend_value(value: Value, op: &crate::op::Op, base: &Path) -> EResult<Value> {
+    match value {
+        Value::String(s) => Ok(extend_string_value(s, op, base).await?),
+        Value::Seq(s) => {
+            let mut result = Vec::with_capacity(s.len());
+            for val in s {
+                result.push(extend_value(val, op, base).await?);
+            }
+            Ok(Value::Seq(result))
+        }
+        Value::Map(m) => {
+            let mut result = BTreeMap::new();
+            for (k, v) in m {
+                result.insert(k, extend_value(v, op, base).await?);
+            }
+            Ok(Value::Map(result))
+        }
+        _ => Ok(value),
+    }
+}
+
+#[cfg(feature = "extended-value")]
+async fn extend_string_value(val: String, op: &crate::op::Op, base: &Path) -> EResult<Value> {
+    if let Some(s) = val.strip_prefix('^') {
+        let mut sp = s.splitn(2, ' ');
+        let cmd = sp.next().unwrap();
+        macro_rules! pipe {
+            () => {{
+                let cmd = sp
+                    .next()
+                    .ok_or_else(|| Error::invalid_params("xvalue pipe: command not specified"))?;
+                let cd_cmd = format!("cd \"{}\" && {}", base.to_string_lossy(), cmd);
+                let res =
+                    bmart::process::command("sh", &["-c", &cd_cmd], op.timeout()?, None, None)
+                        .await?;
+                if res.ok() {
+                    res.out.join("\n")
+                } else {
+                    return Err(Error::failed(format!(
+                        "xvalue pipe command failed to execute: {}",
+                        res.err.join("\n")
+                    )));
+                }
+            }};
+        }
+        match cmd {
+            "include" => {
+                let fname = sp.next().ok_or_else(|| {
+                    Error::invalid_params("xvalue include: file name not specified")
+                })?;
+                let mut path = base.to_path_buf();
+                path.push(fname);
+                let content = tokio::time::timeout(op.timeout()?, tokio::fs::read(path)).await??;
+                let val: Value = serde_yaml::from_slice(&content).map_err(Error::invalid_data)?;
+                Ok(val)
+            }
+            "include-text" => {
+                let fname = sp.next().ok_or_else(|| {
+                    Error::invalid_params("xvalue include: file name not specified")
+                })?;
+                let mut path = base.to_path_buf();
+                path.push(fname);
+                let content =
+                    tokio::time::timeout(op.timeout()?, tokio::fs::read_to_string(path)).await??;
+                Ok(Value::String(content))
+            }
+            "pipe" => {
+                let s = pipe!();
+                let val: Value = serde_yaml::from_str(&s).map_err(Error::invalid_data)?;
+                Ok(val)
+            }
+            "pipe-text" => {
+                let s = pipe!();
+                Ok(Value::String(s))
+            }
+            _ => Ok(Value::String(val)),
+        }
+    } else {
+        Ok(Value::String(val))
+    }
 }
 
 impl Eq for Value {}
